/*
 *
 */
import java.io.*;
import java.util.*;
import java.net.*;
import java.text.*;

public class GetURL {
    public File file;
    public boolean ok=false;
    public int errorCode=200;
    public String contentType;
    public boolean isDirectory=false;
    public boolean isCGI=false;
    private SimpleDateFormat formatter=new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss 'GMT'",new Locale("en","US"));

    public GetURL(HttpRequest httpRequest) {
        String str=httpRequest.httpURL.getFile().replace('/','\\');
        /* Append index.html for subdirs, or if it is missing
         * mark this as a directory to display as a listing
         */
        if(str.lastIndexOf('\\')==str.length()-1) {
            /* We do not list cgi-bin directories! */
            if(str.indexOf("cgi-bin")!=-1) {
                errorCode=404;
                return;
            }
            str=str.concat("index.html");
            file=new File(".\\html"+str);
            /* Support for file listing */
            if(!file.exists()) {
                file=new File(".\\html"+httpRequest.httpURL.getFile().replace('/','\\'));
                ok=true;
                isDirectory=true;
                contentType="text/html";
                return;
            }
        }
        /* Create a file/path pointer. Make sure CGI scripts
         * are properly detected, before changing the path.
         */
        if(str.indexOf("\\cgi-bin")==0) {
            file=new File("."+str);
            isCGI=true;
        } else {
            file=new File(".\\html"+str);
        }
        /* Read file/path information */
        if(file.exists()&file.canRead()) {
            /* Redirection for sub dirs */
            if(file.isDirectory()) {
                errorCode=302;
                return;
            }
            if(httpRequest.ifModifiedSince!=null&str.indexOf("cgi-bin")==-1) {
                try {
                    Date clientDate=formatter.parse(httpRequest.ifModifiedSince,new ParsePosition(0));
                    Date serverDate=new Date(file.lastModified());
                    if(clientDate.before(serverDate)) {
                    } else {
                        errorCode=304;
                        return;
                    }
                } catch(Exception e) {
                    /* If there is an error while parsing the date then
                     * we consider the date field invalid, and proceed
                     * as if this is was a normal request. As defined
                     * by RFC 2616, section 14.25.
                     */
                }
            }
            if(httpRequest.ifUnmodifiedSince!=null&str.indexOf("cgi-bin")==-1) {
                try {
                    Date clientDate=formatter.parse(httpRequest.ifUnmodifiedSince,new ParsePosition(0));
                    Date serverDate=new Date(file.lastModified());
                    if(clientDate.after(serverDate)) {
                    } else {
                        errorCode=412;
                        return;
                    }
                } catch(Exception e) {
                    /* If there is an error while parsing the date then
                     * we consider the date field invalid, and proceed
                     * as if this is was a normal request. As defined
                     * by RFC 2616, section 14.28.
                     */
                }
            }
            ok=true;
            if(file.getName().indexOf('.')!=-1) {
                String ext=file.getName().substring(file.getName().lastIndexOf('.')+1);
                if(ext.equalsIgnoreCase("html"))
                    contentType="text/html";
                else if(ext.equalsIgnoreCase("htm"))
                    contentType="text/html";
                else if(ext.equalsIgnoreCase("png"))
                    contentType="image/png";
                else if(ext.equalsIgnoreCase("gif"))
                    contentType="image/gif";
                else if(ext.equalsIgnoreCase("jpg"))
                    contentType="image/jpeg";
                else
                    contentType="application/octet-stream";
            } else {
                contentType="application/octet-stream";
            }
        } else {
            /* File not found */
            errorCode=404;
            return;
        }
    }

}
