/*
 *
 */
import java.io.*;
import java.util.*;
import java.net.*;
import java.text.*;

public class HttpRequest {

    private Socket sok;
    private BufferedReader input;
    public InetAddress clientAddress;
    public String httpMethod;
    public String httpURI;
    public URL httpURL;
    public double httpVersion=0.9;
    public String host=null;
    public String userAgent;
    public String accept;
    public int keepAlive=-1;
    public String connection="";
    public String ifModifiedSince=null;
    public String ifUnmodifiedSince=null;
    public boolean ok=false;
    public int errorCode=500;

    public HttpRequest(Socket sok) throws Exception {
        this.sok=sok;
        input=new BufferedReader(new InputStreamReader(sok.getInputStream()));
        clientAddress=sok.getInetAddress();
    }

    public void start() throws Exception {
        String str=null;
        /* Start a loop which skips any initial CRLF's based
         * on the suggestion by RFC 2616, section 4.1, max 5 lines. */
        int i=0;
        do {
            i++;
            /* Read the request line, supports timeout exception */
            try { str=input.readLine(); }
            catch(Exception e) { throw new IllegalAccessException(); }
            /* Reject empty requests */
            if(str==null) {
                errorCode=400;
                throw new IOException();
            }
        } while(str.equalsIgnoreCase("")&i<5);
        /* Security policy to ignore large requests. Based
         * on RFC 2616, section 10.4.14.
         */
        if(str.length()>256) {
                errorCode=414;
                throw new IOException();
        }
        /* Parse the elements in the request line based
         * on RFC 2616, section 5.1 */
        StringTokenizer tokens=new StringTokenizer(str);
        if(tokens.countTokens()==2) {
            /* 2 tokens for HTTP/0.9 */
            httpMethod=tokens.nextToken();
            httpURI=URLDecoder.decode(tokens.nextToken());
        } else if(tokens.countTokens()==3) {
            /* 3 tokens for HTTP/1.0 or later */
            httpMethod=tokens.nextToken();
            httpURI=URLDecoder.decode(tokens.nextToken());
            httpVersion=Double.parseDouble(tokens.nextToken().substring(5));
        } else {
            /* Unknown number of tokens in request line */
            errorCode=400;
            throw new IOException();
        }
        /*
         * Reject absolute http requests (http://) which should
         * only be answered by proxy servers. Although RFC 2616,
         * section 5.1.2, suggests otherwise for compatibility with
         * broken or older clients, we still reject these requests
         * to avoid security leaks.
         */
        if(httpURI.length()>=4) {
            if(httpURI.substring(0,4).equalsIgnoreCase("HTTP")) {
                errorCode=400;
                throw new IOException();
            }
        }
        /* Security check for nimda virus and family. */
        if(httpURI.indexOf("/root.exe?/c+")!=-1|httpURI.indexOf("/cmd.exe?/c+")!=-1) {
            errorCode=400;
            /* We drop theconnection right away. Thus avoiding network
             * traffic with evil clients
             */
            throw new IllegalAccessException();
        }
        /* Security check for '..' in the URI path */
        if(httpURI.indexOf("../")!=-1|httpURI.indexOf("/..")!=-1) {
            errorCode=400;
            /* We drop theconnection right away. Thus avoiding network
             * traffic with evil clients
             */
            throw new IllegalAccessException();
        }
        /* Parse the headers */
        while(input.ready()) {
            str=input.readLine();
            if(str==null|str.equals(""))
                break;
            int pos=str.indexOf(":");
            if(pos==-1)
                continue;
            String str2=str.substring(0,pos);
            if(str2.equalsIgnoreCase("Host")) {
                host=str.substring(pos+1).trim();
            } else if(str2.equalsIgnoreCase("User-agent")) {
                userAgent=str.substring(pos+1).trim();
            } else if(str2.equalsIgnoreCase("Accept")) {
                accept=str.substring(pos+1).trim();
            } else if(str2.equalsIgnoreCase("Keep-alive")) {
                keepAlive=Integer.parseInt(str.substring(pos+1).trim());
            } else if(str2.equalsIgnoreCase("Connection")) {
                connection=str.substring(pos+1).trim();
            } else if(str2.equalsIgnoreCase("If-Modified-Since")) {
                ifModifiedSince=str.substring(pos+1).trim();
            } else if(str2.equalsIgnoreCase("If-Unmodified-Since")) {
                ifUnmodifiedSince=str.substring(pos+1).trim();
            }
        }
        /* RFC 2616 requirement, close HTTP/1.1 connections without a Host */
        if(httpVersion>=1.1&host==null) {
            errorCode=400;
            throw new IOException();
        }
        /* RFC 2616 requirement for keep-alive connections. If a client
         * doesn't close the connection, we do so based on timeout delay.
         * Based on RFC 2616, section 8.1.
         */
        if(connection.equalsIgnoreCase("keep-alive")) {
            /* Enable keep-alive */
            sok.setKeepAlive(true);
            /* Set timeout delay */
            if(keepAlive>-1)
                sok.setSoTimeout(1000*keepAlive); /* Supplied by client */
            else
                sok.setSoTimeout(1000*300);       /* Default 300 sec */
        }
        /* Create a URL object from the requested uri */
        if(host!=null)
            httpURL=new URL("http://"+host+httpURI);
        else
            httpURL=new URL("http://"+(InetAddress.getLocalHost().getHostName()).toString()+httpURI);
        /* If we haven't encountered any errors, exit gracefuly */
        if(errorCode==500) {
            ok=true;
            errorCode=200;
        }
    }

    public void reset() throws Exception {
        ok=false;
        errorCode=500;
        httpVersion=0.9;
        host=null;
    }
}
