/*
 *
 */
import java.io.*;
import java.util.*;
import java.net.*;
import java.text.*;

public class HttpResponse {

    private Socket sok;
    private HttpRequest httpRequest;
    private BufferedOutputStream output;

    public HttpResponse(Socket sok,HttpRequest hr) throws IOException {
        this.sok=sok;
        this.httpRequest=hr;
        output=new BufferedOutputStream(new DataOutputStream(sok.getOutputStream()));
    }

    public void sendString(String str) throws IOException {
        output.write(str.getBytes());
    }

    public void sendFile(GetURL file) throws IOException {
        if(file.ok==true) {
            byte[] bytes=new byte[(int)file.file.length()];
            InputStream in=new FileInputStream(file.file);
            int read=0;
            while(read<bytes.length) {
                int len=in.read(bytes,read,bytes.length-read);
                if(len==-1)
                    throw new IOException();
                output.write(bytes,read,len);
                read+=len;
            }
            in.close();
        }
    }

    public void flush() throws IOException {
        output.flush();
    }

    public void sendFileHeaders(GetURL url,int errorCode,double httpVersion) throws IOException {
        String str="HTTP/"+httpVersion+" "+errorCode+" Data follows\r\n";
        SimpleDateFormat formatter=new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss 'GMT'",new Locale("en","US"));
        str=str+"Date: "+formatter.format(new Date())+"\r\n";
        str=str+"Server: JavaTest-HTTPD v1.0\r\n";
        if(url.ok==true) {
            str=str+"Content-Length: "+url.file.length()+"\r\n";
            str=str+"Content-Type: "+url.contentType+"\r\n";
        }
        str=str+"\r\n";
        output.write(str.getBytes());
    }

    public void sendListingHeaders(GetURL url,int errorCode,double httpVersion) throws IOException {
        String str="HTTP/"+httpVersion+" "+errorCode+" Data follows\r\n";
        SimpleDateFormat formatter=new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss 'GMT'",new Locale("en","US"));
        str=str+"Date: "+formatter.format(new Date())+"\r\n";
        str=str+"Server: JavaTest-HTTPD v1.0\r\n";
        str=str+"Connection: close\r\n";
        str=str+"Content-Type: "+url.contentType+"\r\n";
        str=str+"\r\n";
        output.write(str.getBytes());
    }

    public void sendListing(GetURL httpURL) throws IOException {
        String html="<html><head><title>Listing of "+httpURL.file.getName()+"</title></head><body><h2>Listing of "+httpURL.file.getName()+"</h2><pre><hr size=\"1\">\r\n";
        html=html+"Filename<br>\r\n";
        String files[]=httpURL.file.list();
        for(int i=0;i<files.length;i++)
            html=html+"* <a href=\""+files[i]+"\">"+files[i]+"</a>\r\n";
        html=html+"<hr size=\"1\"><b>"+files.length+"</b> entries listed.</pre></body></html>";
        output.write(html.getBytes());
    }

    public void sendCGI(GetURL httpURL,HttpRequest httpRequest,Socket sok) throws IOException {
        /* CGI support is based on the following documentation:
         * "The WWW Common Gateway Interface Version 1.1"
         * http://cgi-spec.golux.com/draft-coar-cgi-v11-03-clean.html
         */
        String env[]= {
            "GATEWAY_INTERFACE="        + "CGI/1.1",
            "REMOTE_ADDR="              + httpRequest.clientAddress.getHostAddress(),
            "REMOTE_HOST="              + httpRequest.clientAddress.getHostName(),
            "REQUEST_METHOD="           + httpRequest.httpMethod,
            "SERVER_NAME="              + InetAddress.getLocalHost().getHostName().toString(),
            "SERVER_PORT="              + sok.getLocalPort(),
            "SERVER_PROTOCOL="          + "HTTP/1.1",
            "SERVER_SOFTWARE="          + "JavaTest-HTTPD v1.0",
            "ETC="                      + System.getProperty("ETC"),
        };
      
        Process p=Runtime.getRuntime().exec(httpURL.file.toString(),env);

        // hook processes input to browser's output (async)
        //InputStream inFromReq=sok.getInputStream();
        //OutputStream outToCgi=p.getOutputStream();

        // hook processes output to browser's input (sync)
        // if browser closes stream, we should detect it and kill process...
        try {
            // read any headers off the top of our input stream
            InputStream in=p.getInputStream();
            // copy remains of input onto output...
            OutputStream out=sok.getOutputStream();

            String str="HTTP/1.1 200 Data follows\r\n";
            out.write(str.getBytes());

            long byteCount=-1;
            int bufferSize = 8192;
            byte buffer[] = new byte[bufferSize];
            int len=bufferSize;
            if(byteCount>=0) {
                while(byteCount>0) {
                    if(byteCount<bufferSize)
                        len=in.read(buffer,0,(int)byteCount);
                    else
                        len=in.read(buffer,0,bufferSize);
                    if (len==-1)
                        break;
                    byteCount -= len;
                    out.write(buffer,0,len);
                }
            } else {
                while (true) {
                    len=in.read(buffer,0,bufferSize);
                    if (len<0 )
                        break;
                    out.write(buffer,0,len);
                }
            }

        } catch (IOException e) {
            // browser has closed its input stream - we should
            // terminate script and clean up...
            p.destroy();
        }

    }
}
